import {
  CheckOutlined,
  EditOutlined,
  SketchOutlined,
  StopOutlined,
  UserSwitchOutlined,
} from "@ant-design/icons";
import {
  Button,
  Card,
  DatePicker,
  Popconfirm,
  Space,
  Table,
  Tooltip,
} from "antd";
import { RangePickerProps } from "antd/es/date-picker";
import dayjs from "dayjs";
import { useState } from "react";
import { selectUserList, updateUser } from "../../api";
import { useMount } from "../../hooks";

interface IEditUser {
  key: string;
  value: any;
}

const defaultEditUser: IEditUser = {
  key: "",
  value: "",
};

const User = () => {
  const [userList, setUserList] = useState([]);
  const [editUser, setEditUser] = useState<IEditUser>(defaultEditUser);

  const onConfirmEditUser = (user: any) => {
    updateUser(user).then((res: any) => {
      if (res?.code === 10000) renderUserList();
    });
  };

  const disabledDate: RangePickerProps["disabledDate"] = (current) => {
    return current && current < dayjs().endOf("day");
  };

  const columns = [
    {
      title: "id",
      dataIndex: "id",
      key: "id",
    },
    {
      title: "用户",
      dataIndex: "user_login",
      key: "user_login",
    },
    {
      title: "创建时间",
      dataIndex: "user_create_time",
      key: "user_create_time",
      render: (_: any, record: any) => {
        return (
          <span>
            {dayjs(+record.user_create_time).format("YYYY-MM-DD HH:mm:ss")}
          </span>
        );
      },
    },
    {
      title: "状态",
      dataIndex: "user_status",
      key: "user_status",
      render: (_: any, record: any) => {
        return record.user_status ? (
          <CheckOutlined style={{ color: "#52c41a" }} />
        ) : (
          "-"
        );
      },
    },
    {
      title: "订阅",
      dataIndex: "user_sub",
      key: "user_sub",
      render: (_: any, record: any) => {
        return record.user_sub ? (
          <CheckOutlined style={{ color: "#52c41a" }} />
        ) : (
          "-"
        );
      },
    },
    {
      title: "订阅到期",
      dataIndex: "user_sub_expired",
      key: "user_sub_expired",
      render: (_: any, record: any) => {
        return record.user_sub ? (
          <Space>
            <span>
              {dayjs(+record.user_sub_expired).format("YYYY-MM-DD HH:mm:ss")}
            </span>
            <Popconfirm
              placement="top"
              title="订阅到期时间"
              description={
                <DatePicker
                  onChange={(date: any) => {
                    setEditUser({
                      key: "user_sub_expired",
                      value: "" + new Date(date).getTime(),
                    });
                  }}
                  presets={[
                    { label: "订阅 - 年", value: dayjs().add(1, "year") },
                    { label: "订阅 - 季度", value: dayjs().add(3, "month") },
                    { label: "订阅 - 月", value: dayjs().add(1, "month") },
                  ]}
                  disabledDate={disabledDate}
                  format="YYYY-MM-DD HH:mm:ss"
                  showTime={{ defaultValue: dayjs("00:00:00", "HH:mm:ss") }}
                />
              }
              onConfirm={() =>
                onConfirmEditUser({ ...record, [editUser.key]: editUser.value })
              }
              okText="确定"
              cancelText="取消"
            >
              <Button type="text" icon={<EditOutlined />} />
            </Popconfirm>
          </Space>
        ) : (
          "-"
        );
      },
    },
    {
      title: "操作",
      dataIndex: "operation",
      key: "operation",
      render: (_: any, record: any) => {
        return (
          <Space>
            <Tooltip title="封号">
              <Button
                type="dashed"
                danger
                icon={<StopOutlined />}
                onClick={() =>
                  onConfirmEditUser({
                    ...record,
                    user_status: !record.user_status,
                  })
                }
              />
            </Tooltip>
            <Tooltip title="订阅">
              <Button
                type="default"
                icon={<UserSwitchOutlined />}
                onClick={() =>
                  onConfirmEditUser({ ...record, user_sub: !record.user_sub })
                }
              />
            </Tooltip>
          </Space>
        );
      },
    },
  ];

  const renderUserList = () => {
    selectUserList().then((res: any) => {
      const { data = [] } = res;
      setUserList(data.map((u: any) => ({ ...u, key: u.id })));
    });
  };

  useMount(() => renderUserList());

  return (
    <div style={{ marginTop: 24 }}>
      <Card>
        <Table dataSource={userList} columns={columns} />
      </Card>
    </div>
  );
};

export default User;
