import {
  Middleware,
  IMiddleware,
  App,
  IMidwayApplication,
} from '@midwayjs/core';
import { NextFunction, Context } from '@midwayjs/koa';
import { BizCode } from '../biz/code';
import { ADMIN, SIGN_DEADLINE, WEB } from '../config/base.config';
import { whiteApis } from '../config/white.api';
import { createToken, decodeToken } from '../util/encrypt';

@Middleware()
export class AuthMiddleware implements IMiddleware<Context, NextFunction> {
  @App()
  app: IMidwayApplication;

  resolve() {
    return async (ctx: Context, next: NextFunction) => {
      const isWhiteApi = whiteApis.some(api => ctx.url.indexOf(api) > -1);
      if (!isWhiteApi) {
        const token = ctx.cookies.get(ADMIN.SIGN) ?? ctx.cookies.get(WEB.SIGN);
        try {
          const { hasLogin, expiredIn, platform, ...rest } = decodeToken(token);
          // token缺少hasLogin
          if (!hasLogin) return { code: BizCode.AUTH, msg: '身份验证错误' };
          // 续签
          const sign = platform === 'web' ? WEB.SIGN : ADMIN.SIGN;
          const signExpired = platform === 'web' ? WEB.EXPIRED : ADMIN.EXPIRED;
          const timeLeft = new Date(expiredIn).getTime() - Date.now();
          if (timeLeft < SIGN_DEADLINE) {
            const expiredIn = new Date(Date.now() + signExpired);
            const token = createToken({
              hasLogin: true,
              platform,
              expiredIn,
              ...rest,
            });
            ctx.cookies.set(sign, token, {
              expires: expiredIn,
              httpOnly: false,
            });
          }
          await next();
        } catch (error) {
          ctx.logger.error(error);
          return { code: BizCode.AUTH, msg: '身份验证错误' };
        }
      } else {
        await next();
      }
    };
  }

  static getName(): string {
    return 'auth';
  }
}
